@@ -126,7 +126,8 @@ volume_get_volume(Slider *s) {
 
 static void 
 volume_show_volume(Slider *s) {
-  gkrellm_update_krell(s->panel,s->krell,volume_get_volume(s));
+  if (s->krell != NULL) 
+    gkrellm_update_krell(s->panel,s->krell,volume_get_volume(s));
   gkrellm_draw_panel_layers(s->panel);
   gkrellm_config_modified();
 }
@@ -218,15 +219,15 @@ volume_toggle_mute(Slider *s) {
 
 static gint 
 bvolume_cb_scroll(GtkWidget *widget, GdkEventScroll *event,Bslider *s) {
-  int amount = s->slider->balance;
+  int amount = 0;
   switch (event->direction) {
     case GDK_SCROLL_UP:
     case GDK_SCROLL_RIGHT:
-      amount += 5;
+      amount = 5;
       break;
     case GDK_SCROLL_DOWN:
     case GDK_SCROLL_LEFT:
-      amount -= 5;
+      amount = -5;
       break;
   }
   volume_set_balance(s->slider,s->slider->balance + amount);
@@ -375,7 +376,6 @@ static void create_bslider(Slider *slide,int first_create) {
                          G_CALLBACK(bvolume_expose_event),result);
     g_signal_connect(G_OBJECT(result->panel->drawing_area), "scroll_event", 
                          G_CALLBACK(bvolume_cb_scroll), result);
-
     g_signal_connect(G_OBJECT(result->panel->drawing_area),"button_press_event",
                         G_CALLBACK(bvolume_button_press),result);
     g_signal_connect(GTK_OBJECT(result->panel->drawing_area),
@@ -387,45 +387,68 @@ static void create_bslider(Slider *slide,int first_create) {
   volume_show_balance(slide);
 }
 
+void 
+toggle_button_press(GkrellmDecalbutton *button, Slider *s) {
+  int l,r ;
+  mixer_get_device_volume(s->mixer, s->dev, &l, &r);
+  mixer_set_device_volume(s->mixer, s->dev, ++l % 2, ++r % 2);
+}
+
 static void create_slider(Slider *s,int first_create) {
   GkrellmStyle *panel_style = gkrellm_meter_style(VOLUME_STYLE);
   GkrellmStyle *slider_style = 
     gkrellm_copy_style(gkrellm_meter_style_by_name("volume.level_slider"));
   GkrellmPiximage *krell_image;
 
+  /* Switches not supported yet ! */
+  if (mixer_get_device_fullscale(s->mixer, s->dev) == 1) return;
+
   gkrellm_set_style_slider_values_default(slider_style,0,0,0);
 
   if (first_create) s->panel = gkrellm_panel_new0();
 
-  krell_image = gkrellm_krell_slider_piximage();
-  s->krell = gkrellm_create_krell(s->panel,krell_image,slider_style);
-
-  gkrellm_set_krell_full_scale(s->krell, 
-                               mixer_get_device_fullscale(s->mixer,s->dev), 1);
-  gkrellm_monotonic_krell_values(s->krell, FALSE);
 
   gkrellm_panel_configure(s->panel,
                           mixer_get_device_name(s->mixer,s->dev),
                           panel_style);
   gkrellm_panel_create(pluginbox, monitor, s->panel);
   /* center the krell if the style is not themed */
-  if (!gkrellm_style_is_themed(slider_style,GKRELLMSTYLE_KRELL_YOFF))
-    gkrellm_move_krell_yoff(s->panel, 
-        s->krell,(s->panel->h - s->krell->h_frame) / 2);
+  if (mixer_get_device_fullscale(s->mixer, s->dev) != 1) {
+    krell_image = gkrellm_krell_slider_piximage();
+    s->krell = gkrellm_create_krell(s->panel,krell_image,slider_style);
+    gkrellm_set_krell_full_scale(s->krell, 
+                               mixer_get_device_fullscale(s->mixer,s->dev), 1);
+    gkrellm_monotonic_krell_values(s->krell, FALSE);
 
-  if (first_create) {
-     g_signal_connect(GTK_OBJECT(s->panel->drawing_area), "expose_event",
-                         G_CALLBACK(volume_expose_event),s);
-     g_signal_connect(G_OBJECT(s->panel->drawing_area),
-                         "scroll_event", G_CALLBACK(volume_cb_scroll), s);
-     g_signal_connect(G_OBJECT(s->panel->drawing_area), "button_press_event", 
-                      G_CALLBACK(volume_button_press),s);
-     g_signal_connect(GTK_OBJECT(s->panel->drawing_area),"button_release_event",
-                      G_CALLBACK(volume_button_release),s);
-     g_signal_connect(GTK_OBJECT(s->panel->drawing_area),"motion_notify_event",
+    if (!gkrellm_style_is_themed(slider_style,GKRELLMSTYLE_KRELL_YOFF))
+      gkrellm_move_krell_yoff(s->panel, 
+          s->krell,(s->panel->h - s->krell->h_frame) / 2);
+
+    if (first_create) {
+       g_signal_connect(G_OBJECT(s->panel->drawing_area),
+                           "scroll_event", G_CALLBACK(volume_cb_scroll), s);
+       g_signal_connect(G_OBJECT(s->panel->drawing_area), "button_press_event", 
+                        G_CALLBACK(volume_button_press),s);
+       g_signal_connect(GTK_OBJECT(s->panel->drawing_area),
+                        "button_release_event",
+                        G_CALLBACK(volume_button_release),s);
+       g_signal_connect(GTK_OBJECT(s->panel->drawing_area),
+                       "motion_notify_event",
                       G_CALLBACK(volume_motion),s);
+    }
+  } else {
+    g_assert_not_reached();
+    s->button = gkrellm_make_scaled_button(s->panel, NULL,
+                                           toggle_button_press, s, 
+                                           FALSE, FALSE, 0, 0, 0,
+                                           gkrellm_chart_width() - 15, 0,
+                                           10, 0 );
   }
 
+  if (first_create) {
+    g_signal_connect(GTK_OBJECT(s->panel->drawing_area), "expose_event",
+      G_CALLBACK(volume_expose_event),s);
+  }
   volume_show_volume(s);
   if (GET_FLAG(s->flags,BALANCE)) create_bslider(s,first_create);
 }
@@ -596,6 +619,75 @@ device_name_edited(GtkCellRendererText *text,
   mixer_config_changed = TRUE;
 }
 
+static void up_clicked(GtkWidget *widget,gpointer user_data) {
+  GtkTreeIter selected,up,new;
+  GtkTreeView *view = GTK_TREE_VIEW(user_data);
+  GtkTreeSelection *selection;
+  GtkTreeModel *model;
+  GtkTreePath *path;
+  gchar *name,*id;
+  gpointer *child_model,*nb;
+
+  selection = gtk_tree_view_get_selection(view);
+
+  if (!gtk_tree_selection_get_selected(selection,&model,&selected)) return;
+  gtk_tree_model_get(model,&selected,
+       ID_COLUMN,&id,
+       NAME_COLUMN,&name,
+       C_MODEL_COLUMN,&child_model,
+       C_NB_COLUMN,&nb,
+        -1);
+
+  path = gtk_tree_model_get_path(model,&selected);
+  if (!gtk_tree_path_prev(path)) return;
+  if (!gtk_tree_model_get_iter(model,&up,path)) return;
+  gtk_list_store_remove(GTK_LIST_STORE(model),&selected);
+  gtk_list_store_insert_before(GTK_LIST_STORE(model),&new,&up);
+
+  gtk_list_store_set(GTK_LIST_STORE(model),&new,
+       ID_COLUMN,id,
+       NAME_COLUMN,name,
+       C_MODEL_COLUMN,child_model,
+       C_NB_COLUMN,nb,
+       -1);
+  mixer_config_changed = TRUE;
+
+}
+
+static void down_clicked(GtkWidget *widget,gpointer user_data) {
+  GtkTreeIter selected,down,new;
+  GtkTreeView *view = GTK_TREE_VIEW(user_data);
+  GtkTreeSelection *selection;
+  GtkTreeModel *model;
+  GtkTreePath *path;
+  gchar *name,*id;
+  gpointer *child_model,*nb;
+
+  selection = gtk_tree_view_get_selection(view);
+
+  if (!gtk_tree_selection_get_selected(selection,&model,&selected)) return;
+  gtk_tree_model_get(model,&selected,
+       ID_COLUMN,&id,
+       NAME_COLUMN,&name,
+       C_MODEL_COLUMN,&child_model,
+       C_NB_COLUMN,&nb,
+        -1);
+
+  path = gtk_tree_model_get_path(model,&selected);
+  gtk_tree_path_next(path);
+  if (!gtk_tree_model_get_iter(model,&down,path)) return;
+  gtk_list_store_insert_after(GTK_LIST_STORE(model),&new,&down);
+
+  gtk_list_store_set(GTK_LIST_STORE(model),&new,
+       ID_COLUMN,id,
+       NAME_COLUMN,name,
+       C_MODEL_COLUMN,child_model,
+       C_NB_COLUMN,nb,
+       -1);
+  mixer_config_changed = TRUE;
+  gtk_list_store_remove(GTK_LIST_STORE(model),&selected);
+}
+
 static GtkWidget *
 create_device_notebook(GtkListStore *store, char *name) {
   GtkWidget *treeview;
@@ -603,6 +695,8 @@ create_device_notebook(GtkListStore *store, char *name) {
   GtkWidget *scrolledwindow;
   GtkWidget *label;
   GtkWidget *vbox;
+  GtkWidget *hbox;
+  GtkWidget *widget;
   GtkWidget *topvbox;
   
   topvbox = gtk_vbox_new(FALSE,0);
@@ -673,8 +767,21 @@ create_device_notebook(GtkListStore *store, char *name) {
                      GTK_POLICY_ALWAYS);
 
 
+  hbox = gtk_hbox_new(FALSE, 3);
+  widget = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
+  g_signal_connect(G_OBJECT(widget),"clicked",
+                   G_CALLBACK(up_clicked),treeview);
+  gtk_box_pack_start(GTK_BOX(hbox),widget,FALSE,FALSE,3);
+
+  widget = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
+  g_signal_connect(G_OBJECT(widget),"clicked",
+                   G_CALLBACK(down_clicked),treeview);
+  gtk_box_pack_start(GTK_BOX(hbox),widget,FALSE,FALSE,3);
+
+
   gtk_box_pack_start(GTK_BOX(vbox),scrolledwindow,TRUE,TRUE,3);
   gtk_container_add(GTK_CONTAINER(scrolledwindow),treeview);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 3);
 
   gtk_widget_show_all(topvbox);
   return topvbox;
@@ -697,11 +804,15 @@ static void add_mixer_to_model(char *id, mixer_t *mixer, Slider *s) {
       );
 
    for(i = 0; i < mixer_get_nr_devices(mixer); i++) {
+     if (mixer_get_device_fullscale(mixer, i) == 1) {
+       /*Switch not supported yet */
+       continue;
+     }
      if (s != NULL && s->dev == i) { 
        enabled = TRUE;
        save_volume = GET_FLAG(s->flags,SAVE_VOLUME);
        balance = GET_FLAG(s->flags,BALANCE);
-         s = s->next;
+       s = s->next;
       } else {
         enabled = save_volume = balance = FALSE;
       }
@@ -764,76 +875,6 @@ static void add_mixerid_to_model(char *id,gboolean gui) {
   mixer_close(mixer);
   return;
 }
-
-static void up_clicked(GtkWidget *widget,gpointer user_data) {
-  GtkTreeIter selected,up,new;
-  GtkTreeView *view = GTK_TREE_VIEW(user_data);
-  GtkTreeSelection *selection;
-  GtkTreeModel *model;
-  GtkTreePath *path;
-  gchar *name,*id;
-  gpointer *child_model,*nb;
-
-  selection = gtk_tree_view_get_selection(view);
-
-  if (!gtk_tree_selection_get_selected(selection,&model,&selected)) return;
-  gtk_tree_model_get(model,&selected,
-       ID_COLUMN,&id,
-       NAME_COLUMN,&name,
-       C_MODEL_COLUMN,&child_model,
-       C_NB_COLUMN,&nb,
-        -1);
-
-  path = gtk_tree_model_get_path(model,&selected);
-  if (!gtk_tree_path_prev(path)) return;
-  if (!gtk_tree_model_get_iter(model,&up,path)) return;
-  gtk_list_store_remove(GTK_LIST_STORE(model),&selected);
-  gtk_list_store_insert_before(GTK_LIST_STORE(model),&new,&up);
-
-  gtk_list_store_set(GTK_LIST_STORE(model),&new,
-       ID_COLUMN,id,
-       NAME_COLUMN,name,
-       C_MODEL_COLUMN,child_model,
-       C_NB_COLUMN,nb,
-       -1);
-  mixer_config_changed = TRUE;
-
-}
-
-static void down_clicked(GtkWidget *widget,gpointer user_data) {
-  GtkTreeIter selected,down,new;
-  GtkTreeView *view = GTK_TREE_VIEW(user_data);
-  GtkTreeSelection *selection;
-  GtkTreeModel *model;
-  GtkTreePath *path;
-  gchar *name,*id;
-  gpointer *child_model,*nb;
-
-  selection = gtk_tree_view_get_selection(view);
-
-  if (!gtk_tree_selection_get_selected(selection,&model,&selected)) return;
-  gtk_tree_model_get(model,&selected,
-       ID_COLUMN,&id,
-       NAME_COLUMN,&name,
-       C_MODEL_COLUMN,&child_model,
-       C_NB_COLUMN,&nb,
-        -1);
-
-  path = gtk_tree_model_get_path(model,&selected);
-  gtk_tree_path_next(path);
-  if (!gtk_tree_model_get_iter(model,&down,path)) return;
-  gtk_list_store_insert_after(GTK_LIST_STORE(model),&new,&down);
-
-  gtk_list_store_set(GTK_LIST_STORE(model),&new,
-       ID_COLUMN,id,
-       NAME_COLUMN,name,
-       C_MODEL_COLUMN,child_model,
-       C_NB_COLUMN,nb,
-       -1);
-  mixer_config_changed = TRUE;
-  gtk_list_store_remove(GTK_LIST_STORE(model),&selected);
-}
-
 #ifndef WIN32
 static void file_choosen(GtkWidget *w,gpointer selector) {
   char *id;
